import struct

# Data types
_FLAGS = 1
_SHORTENED_LOCAL_NAME = 8
_SERVICE_DATA_UUID16 = 0x16

# Service data UUID
_BTHOME = 0xFCD2

# Not encrypted, not trigger based, BTHome v2
_DEVICE_INFORMATION = 0x40

_BATTERY = 0x01  # % uint8
_TEMPERATURE = 0x02  # centi-degrees int16
_HUMIDITY = 0x03  # centi-percent uint16
_CO2 = 0x12  # PPM uint16
_VOLTAGE = 0x0C  # mV uint16


def _pack(type: int, element: bytes) -> bytes:
    return bytes((len(element) + 1, type)) + element


def _pack_int16(id: int, value: float) -> bytes:
    return struct.pack("<Bh", id, int(value))


def _pack_uint16(id: int, value: float) -> bytes:
    return struct.pack("<BH", id, int(value))


def _pack_uint8(id: int, value: float) -> bytes:
    return struct.pack("<BB", id, int(value))


def pack(
    name: str,
    temperature: float | None = None,
    humidity_ratio: float | None = None,
    co2: float | None = None,
    battery_ratio: float | None = None,
    battery_v: float | None = None,
):
    payload = b"\x40"
    if temperature is not None:
        payload += _pack_int16(_TEMPERATURE, temperature * 100)
    if humidity_ratio is not None:
        payload += _pack_uint16(_HUMIDITY, humidity_ratio * 100 * 100)
    if co2 is not None:
        payload += _pack_uint16(_CO2, co2)
    if battery_ratio is not None:
        payload += _pack_uint8(_BATTERY, battery_ratio * 100)
    if battery_v is not None:
        payload += _pack_uint16(_VOLTAGE, battery_v * 1000)

    return (
        _pack(_FLAGS, b"\x06")
        + _pack(_SERVICE_DATA_UUID16, struct.pack("<H", _BTHOME) + payload)
        + _pack(_SHORTENED_LOCAL_NAME, name.encode("utf-8"))
    )
